@@ -13,7 +13,7 @@ import subprocess
 from mash import io_util
 from mash.filesystem.filesystem import FileSystem
 from mash.io_util import log, shell_ready_signal, print_shell_ready_signal, check_output
-from mash.util import for_any, has_method, identity, is_alpha, is_globbable, omit_prefixes, split_prefixes, split_sequence, glob
+from mash.util import for_any, has_method, identity, is_alpha, is_globbable, is_valid_method_name, omit_prefixes, split_prefixes, split_sequence, glob
 from mash.shell.function import InlineFunction
 
 confirmation_mode = False
@@ -75,6 +75,7 @@ class BaseShell(Cmd):
 
         self.env = {}
         self.locals = FileSystem(defaultdict(dict))
+
         self.update_env(env)
 
         self.auto_save = False
@@ -91,13 +92,18 @@ class BaseShell(Cmd):
 
     @property
     def delimiters(self):
-        # Return the latest values of these lists
-        return py_delimiters + bash_delimiters + ['->']
+        """Return the most recent values of the delimiters.
+        """
+        delimiters = py_delimiters + bash_delimiters + ['->']
+
+        # insert '<-' after ';'
+        delimiters.insert(1, '<-')
+
+        return delimiters
 
     def set_infix_operators(self):
         # use this for infix operators, e.g. `a = 1`
-        self.infix_operators = {'=': self.handle_equation,
-                                '<-': self.eval_and_set_env_variable}
+        self.infix_operators = {'=': self.handle_equation}
         # the sign to indicate that a variable should be expanded
         self.variable_prefix = '$'
 
@@ -160,21 +166,23 @@ class BaseShell(Cmd):
 
     def _retrieve_eval_result(self, k):
         if k in self.env:
-            return self.env[k]
+            return str(self.env[k])
 
         elif self._last_results:
             return self._last_results.pop()
 
         raise RuntimeError('Cannot retrieve result')
 
-    def handle_equation(self, lhs: Tuple[str], *rhs: str):
+    def handle_equation(self, lhs: Tuple[str], *rhs: str) -> str:
         if len(lhs) == 1:
             k = lhs[0]
-            return self.set_env_variable(k, *rhs)
+            self.set_env_variable(k, *rhs)
+            return ''
 
         f, *args = lhs
         implementation = ' '.join(rhs[1:])
-        return self.add_inline_function(f, args, implementation)
+        self.add_inline_function(f, args, implementation)
+        return ''
 
     def add_inline_function(self, f, args, inner) -> str:
         if has_method(self, f'do_{f}'):
@@ -182,6 +190,9 @@ class BaseShell(Cmd):
                 f'Name conflict: Cannot define inline function {f}, '
                 f'because there already exists a method do_{f}.')
 
+        if not is_valid_method_name(f):
+            raise ShellError(f'Invalid function name format: {f}')
+
         inner = self.expand_variables_inline(inner)
 
         # TODO use custom class with attr .functions instead of a string
@@ -189,7 +200,6 @@ class BaseShell(Cmd):
 
         positionals = ' '.join(args)
         log(f'function {f}({positionals});')
-        return ''
 
     def call_inline_function(self, f: InlineFunction, *args: str):
         translations = {}
@@ -211,23 +221,13 @@ class BaseShell(Cmd):
     def set_env_variable(self, k: str, *values: str):
         """Set the variable `k` to `values`
         """
+        if not is_valid_method_name(k):
+            raise ShellError(f'Invalid variable name format: {k}')
+
+        log(f'set {k}')
         self.env[k] = ' '.join(values)
         return k
 
-    def eval_and_set_env_variable(self, k: Tuple[str], *values: str):
-        """Evaluate `values` as an expression and store the result in the variable `k`
-        """
-        k: str = k[0]
-
-        try:
-            result = self.eval(values)
-        except ShellError:
-            log(f'Error, cannot set {k}')
-            return
-
-        self.env[k] = result
-        return k
-
     def show_env(self, env=None):
         if env is None:
             env = self.env
@@ -302,6 +302,23 @@ class BaseShell(Cmd):
     # Commands: do_*
     ############################################################################
 
+    def do_assign(self, args: str):
+        """Assign the result of an expression to an environment variable.
+        ```sh
+        assign a |> print 10
+        # results in a = 10
+        ```
+        """
+        k = args.split(' ')[0]
+
+        if not is_valid_method_name(k):
+            raise ShellError('Invalid variable name format: {k}')
+
+        self.locals.set('assignee', k)
+
+        # return value must be empty to prevent side-effects in the next command
+        return ''
+
     def do_export(self, args: str):
         """Set an environment variable.
         `export(k, *values)`
@@ -316,10 +333,9 @@ class BaseShell(Cmd):
             if k in self.env:
                 del self.env[k]
             else:
-                logging.warn('Invalid key')
+                logging.warning('Invalid key')
             return
 
-        log(f'set {k}')
         self.set_env_variable(k, *values)
 
     def do_shell(self, args):
@@ -416,7 +432,11 @@ class BaseShell(Cmd):
         """
         try:
             line = self.onecmd_prehook(line)
-            lines = self.parse_commands(line)
+            lines = list(self.parse_commands(line))
+
+            if any(line[0] == '<-' for line in lines):
+                lines[0] = ['assign'] + lines[0]
+
             self.run_commands(lines)
 
         except CancelledError:
@@ -490,7 +510,10 @@ class BaseShell(Cmd):
 
                 raise ShellError(str(e))
 
-        if result is not None:
+        if 'assignee' in self.locals:
+            self._save_assignee(result)
+
+        elif result is not None:
             print(result)
 
     def run_single_command(self, command_and_args: List[str], result: str = '') -> str:
@@ -512,17 +535,32 @@ class BaseShell(Cmd):
             elif prefixes[-1] == '->':
                 # TODO verify syntax
                 assert ' ' not in line
-                return self.set_env_variable(line, result)
+                self.set_env_variable(line, result)
+                return ''
 
         if infix_operator_args:
             return self.infix_command(*infix_operator_args)
 
         return self.pipe_cmd_py(line, result)
 
+    def _save_assignee(self, result: str):
+        k = self.locals['assignee']
+
+        if result is None:
+            raise ShellError(f'Missing return value in assignment: {k}')
+        elif result == '' and self._last_results:
+            result = self._last_results.pop()
+
+        self.env[k] = result
+        self.locals.rm('assignee')
+
     def filter_result(self, command_and_args, result):
         if ';' in command_and_args:
-            # print prev result & discard it
-            if result is not None:
+            if 'assignee' in self.locals:
+                self._save_assignee(result)
+
+            elif result is not None:
+                # print prev result & discard it
                 print(result)
 
             result = ''
@@ -675,7 +713,7 @@ class BaseShell(Cmd):
                 error_msg = f'Variable `{v}` is not set'
 
                 if k in self.env:
-                    yield self.env[k]
+                    yield str(self.env[k])
                     continue
                 elif self.ignore_invalid_syntax:
                     log(error_msg)
